use mpd;
use mpd::status::State;
use mpd::idle;
use mpd::idle::Idle;
use std::thread;
use ui::widget;
use ui::widget::Widget;

const MARGIN: u16 = 7;
const WIDTH: u16 = 250;

pub struct Music {
    context: widget::DrawContext,
    tx: widget::MessageSender,
    state: State,
    artist: String,
    title: String,
    last_pos: u16
}

pub fn mpd(tx: widget::MessageSender, context: widget::DrawContext) -> Music {
    Music {
        context: context,
        tx: tx,
        state: State::Stop,
        artist: "".to_string(),
        title: "".to_string(),
        last_pos: 0
    }
}

impl Music {
    fn icon(&self) -> &str {
        match self.state {
            State::Play => "",
            State::Pause => "",
            State::Stop => ""
        }
    }

    fn redraw(&mut self) {
        let x = self.last_pos;
        self.context.set_bg_color(0x0, 0x0, 0x0, 0xFFFF);
        self.context.draw_bg(x, WIDTH);
        self.render(x);
        self.context.flush();
    }

}

impl widget::Widget for Music {
    fn init(&mut self) {
        let tx = self.tx.clone();
        thread::spawn(move || monitor_thread(tx));
    }

    fn render(&mut self, x: u16) {
        self.last_pos = x;
        let icon_width = self.context.measure_text(self.icon());

        self.context.set_bg_color(0x6666, 0x6666, 0x6666, 0xFFFF);
        self.context.draw_bg(x, icon_width + MARGIN * 2);
        self.context.draw_text(self.icon(), x + MARGIN);


        let text = if self.state == State::Stop {
            "Stopped".to_string()
        }
        else {
            format!("{} - {}", self.artist, self.title)
        };
        let text_width = WIDTH - MARGIN * 4 - icon_width;
        self.context.set_bg_color(0x0, 0x0, 0x0, 0xFFFF);
        self.context.draw_bg(x + icon_width + MARGIN * 3, text_width);
        self.context.draw_text_until(&text, x + icon_width + MARGIN * 3, text_width);
    }

    fn width(&mut self) -> u16 {
        WIDTH
    }

    fn handle_event(&mut self, event: &widget::Message) -> bool {
        match event {
            &widget::Message::MpdEvent(ref state, ref artist, ref title) => {
                self.state = state.clone();
                self.artist = artist.clone();
                self.title = title.clone();
                self.redraw();
            },
            &widget::Message::MousePress(_x) => {
                toggle();
            },
            _ => {}
        }
        false
    }
}

fn monitor_thread(tx: widget::MessageSender) {
    let mut conn = mpd::client::Client::connect("127.0.0.1:6600").unwrap();
    loop {
        let unknown = "Unknown".to_string();
        let state = conn.status().unwrap().state;
        let song = conn.currentsong().unwrap().unwrap();
        let artist = song.tags.get("Artist").unwrap_or(&unknown);
        let title = song.title.as_ref().unwrap_or(&unknown);

        tx.send(widget::Message::MpdEvent(state, artist.clone(), title.clone())).expect("Failed to send mpd event");
        conn.wait(&[idle::Subsystem::Player]).ok();
    }
}

fn toggle() {
    let mut conn = mpd::client::Client::connect("127.0.0.1:6600").unwrap();
    conn.toggle_pause().expect("Failed to pause");
}
